@@ -1,3 +1,19 @@
+# eopayment - online payment library
+# Copyright (C) 2011-2020 Entr'ouvert
+#
+# This program is free software: you can redistribute it and/or modify it
+# under the terms of the GNU Affero General Public License as published
+# by the Free Software Foundation, either version 3 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU Affero General Public License for more details.
+#
+# You should have received a copy of the GNU Affero General Public License
+# along with this program.  If not, see <http://www.gnu.org/licenses/>.
+
 import os.path
 import os
 import random
@@ -54,6 +70,7 @@ def force_text(s, encoding='utf-8'):
         return six.text_type(s, encoding, 'ignore')
     return s
 
+
 def force_byte(s, encoding='utf-8'):
     if isinstance(s, bytes):
         return s
@@ -94,7 +111,7 @@ class PaymentResponse(object):
 
     def __init__(self, result=None, signed=None, bank_data=dict(),
                  return_content=None, bank_status='', transaction_id='',
-                 order_id='', test=False):
+                 order_id='', test=False, transaction_date=None):
         self.result = result
         self.signed = signed
         self.bank_data = bank_data
@@ -103,6 +120,7 @@ class PaymentResponse(object):
         self.transaction_id = transaction_id
         self.order_id = order_id
         self.test = test
+        self.transaction_date = transaction_date
 
     def __repr__(self):
         return '<%s %s>' % (self.__class__.__name__, repr(self.__dict__))
@@ -143,7 +161,7 @@ class PaymentCommon(object):
             try:
                 fd = os.open(os.path.join(self.PATH, name),
                              os.O_CREAT | os.O_EXCL)
-            except:
+            except Exception:
                 raise
             else:
                 os.close(fd)
