@@ -1,4 +1,19 @@
 # -*- coding: utf-8
+# eopayment - online payment library
+# Copyright (C) 2011-2020 Entr'ouvert
+#
+# This program is free software: you can redistribute it and/or modify it
+# under the terms of the GNU Affero General Public License as published
+# by the Free Software Foundation, either version 3 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU Affero General Public License for more details.
+#
+# You should have received a copy of the GNU Affero General Public License
+# along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
 import codecs
 from collections import OrderedDict
@@ -7,6 +22,9 @@ import logging
 import hashlib
 import hmac
 import requests
+
+import pytz
+
 from decimal import Decimal, ROUND_DOWN
 from Crypto.Signature import PKCS1_v1_5
 from Crypto.PublicKey import RSA
@@ -22,7 +40,8 @@ import string
 import warnings
 
 from .common import (PaymentCommon, PaymentResponse, FORM, PAID, ERROR, Form,
-        ORDERID_TRANSACTION_SEPARATOR, ResponseError, force_text, force_byte)
+                     ORDERID_TRANSACTION_SEPARATOR, ResponseError, force_text,
+                     force_byte)
 
 __all__ = ['sign', 'Payment']
 
@@ -39,7 +58,7 @@ VARS = {
     'PBX_IDENTIFIANT': 'Identifiant interne (fourni par Paybox)',
     'PBX_TOTAL': 'Montant total de la transaction',
     'PBX_DEVISE': 'Devise de la transaction',
-    'PBX_CMD':  'Référence commande côté commerçant',
+    'PBX_CMD': 'Référence commande côté commerçant',
     'PBX_PORTEUR': 'Adresse E - mail de l’acheteur',
     'PBX_RETOUR': 'Liste des variables à retourner par Paybox',
     'PBX_HASH': 'Type d’algorit hme de hachage pour le calcul de l’empreinte',
@@ -52,7 +71,7 @@ PAYBOX_ERROR_CODES = {
     '00001': 'La connexion au centre d’autorisation a échoué ou une '
     'erreur interne est survenue. Dans ce cas, il est souhaitable de faire '
     'une tentative sur le site secondaire : tpeweb1.paybox.com.',
-    '001xx':  'Paiement refusé par le centre d’autorisation [voir '
+    '001xx': 'Paiement refusé par le centre d’autorisation [voir '
     '§12.112.1 Codes réponses du centre d’autorisationCodes réponses du '
     'centre d’autorisation]. En cas d’autorisation de la transaction par '
     'le centre d’autorisation de la banque ou de l’établissement financier '
@@ -182,41 +201,41 @@ class Payment(PaymentCommon):
                 'name': 'platform',
                 'caption': _('Plateforme cible'),
                 'default': 'test',
-                'validation': lambda x: isinstance(x, basestring) and
-                x.lower() in ('test', 'prod'),
+                'validation': lambda x: isinstance(x, six.string_types)
+                and x.lower() in ('test', 'prod'),
             },
             {
                 'name': 'site',
                 'caption': _('Numéro de site'),
                 'required': True,
-                'validation': lambda x: isinstance(x, basestring) and
-                x.isdigit() and len(x) == 7,
+                'validation': lambda x: isinstance(x, six.string_types)
+                and x.isdigit() and len(x) == 7,
             },
             {
                 'name': 'cle',
                 'caption': _('Site key'),
                 'required': False,
-                'validation': lambda x: isinstance(x, basestring),
+                'validation': lambda x: isinstance(x, six.string_types),
             },
             {
                 'name': 'rang',
                 'caption': _('Numéro de rang'),
                 'required': True,
-                'validation': lambda x: isinstance(x, basestring) and
-                x.isdigit() and len(x) == 2,
+                'validation': lambda x: isinstance(x, six.string_types)
+                and x.isdigit() and len(x) == 2,
             },
             {
                 'name': 'identifiant',
                 'caption': _('Identifiant'),
                 'required': True,
-                'validation': lambda x: isinstance(x, basestring) and
-                x.isdigit() and (0 < len(x) < 10),
+                'validation': lambda x: isinstance(x, six.string_types)
+                and x.isdigit() and (0 < len(x) < 10),
             },
             {
                 'name': 'shared_secret',
                 'caption': _('Secret partagé'),
-                'validation': lambda x: isinstance(x, str) and
-                all(a.lower() in '0123456789ABCDEF' for a in x),
+                'validation': lambda x: isinstance(x, str)
+                and all(a.lower() in '0123456789ABCDEF' for a in x),
                 'required': True,
             },
             {
@@ -237,8 +256,8 @@ class Payment(PaymentCommon):
                 'caption': _('Nombre de jours pour un paiement différé'),
                 'default': '',
                 'required': False,
-                'validation': lambda x: isinstance(x, basestring) and
-                x.isdigit() and (1 <= len(x) <= 2)
+                'validation': lambda x: isinstance(x, six.string_types)
+                and x.isdigit() and (1 <= len(x) <= 2)
             },
             {
                 'name': 'capture_mode',
@@ -253,6 +272,12 @@ class Payment(PaymentCommon):
                 'type': bool,
                 'default': False,
                 'scope': 'transaction'
+            },
+            {
+                'name': 'timezone',
+                'caption': _('Default Timezone'),
+                'default': 'Europe/Paris',
+                'required': False,
             }
         ]
     }
@@ -272,9 +297,16 @@ class Payment(PaymentCommon):
         if orderid:
             d['PBX_CMD'] = orderid + ORDERID_TRANSACTION_SEPARATOR + d['PBX_CMD']
         d['PBX_PORTEUR'] = force_text(email)
-        d['PBX_RETOUR'] = 'montant:M;reference:R;code_autorisation:A;erreur:E;numero_appel:T;numero_transaction:S;signature:K'
+        d['PBX_RETOUR'] = (
+            'montant:M;reference:R;code_autorisation:A;erreur:E;numero_appel:T;'
+            'numero_transaction:S;'
+            'date_transaction:W;heure_transaction:Q;'
+            'signature:K'
+        )
         d['PBX_HASH'] = 'SHA512'
-        d['PBX_TIME'] = kwargs.get('time') or (force_text(datetime.datetime.utcnow().isoformat('T')).split('.')[0]+'+00:00')
+        d['PBX_TIME'] = kwargs.get('time') or (
+            force_text(datetime.datetime.utcnow().isoformat('T')).split('.')[0]
+            + '+00:00')
         d['PBX_ARCHIVAGE'] = transaction_id
         if self.normal_return_url:
             d['PBX_EFFECTUE'] = self.normal_return_url
@@ -324,7 +356,8 @@ class Payment(PaymentCommon):
             data = []
             if callback:
                 for key in ('montant', 'reference', 'code_autorisation',
-                            'erreur', 'numero_appel', 'numero_transaction'):
+                            'erreur', 'numero_appel', 'numero_transaction',
+                            'date_transaction', 'heure_transaction'):
                     data.append('%s=%s' % (key, urllib.quote(d[key][0])))
             else:
                 for key, value in urlparse.parse_qsl(query_string, True, True):
@@ -339,7 +372,7 @@ class Payment(PaymentCommon):
             result = ERROR
         for l in (5, 3):
             prefix = d['erreur'][0][:l]
-            suffix = 'x' * (5-l)
+            suffix = 'x' * (5 - l)
             bank_status = PAYBOX_ERROR_CODES.get(prefix + suffix)
             if bank_status is not None:
                 break
@@ -347,28 +380,42 @@ class Payment(PaymentCommon):
         # decode order id from returned reference
         if ORDERID_TRANSACTION_SEPARATOR in orderid:
             orderid, transaction_id = orderid.split(ORDERID_TRANSACTION_SEPARATOR, 1)
+        transaction_date = None
+        if 'date_transaction' in d and 'heure_transaction' in d:
+            try:
+                full_date_string = '%sT%s' % (d['date_transaction'][0], d['heure_transaction'][0])
+                transaction_date = datetime.datetime.strptime(full_date_string, '%Y%m%dT%H:%M:%S')
+            except ValueError:
+                pass
+            else:
+                # We suppose Europe/Paris is the default timezone for Paybox
+                # servers.
+                paris_tz = pytz.timezone(self.timezone)
+                transaction_date = paris_tz.localize(transaction_date)
         return PaymentResponse(
             order_id=orderid,
             signed=signed,
             bank_data=d,
             result=result,
-            bank_status=bank_status)
+            bank_status=bank_status,
+            transaction_date=transaction_date)
 
     def perform(self, amount, bank_data, operation):
         logger = logging.getLogger(__name__)
         url = PAYBOX_DIRECT_URLS[self.platform]
-        params = {'VERSION': PAYBOX_DIRECT_VERSION_NUMBER,
-                  'TYPE': operation,
-                  'SITE': force_text(self.site),
-                  'RANG': self.rang.strip(),
-                  'CLE': force_text(self.cle),
-                  'NUMQUESTION': bank_data['numero_transaction'][0].zfill(10),
-                  'MONTANT': (amount * Decimal(100)).to_integral_value(ROUND_DOWN),
-                  'DEVISE': force_text(self.devise),
-                  'NUMTRANS': bank_data['numero_transaction'][0], # paybox transaction number
-                  'NUMAPPEL': bank_data['numero_appel'][0],
-                  'REFERENCE': bank_data['reference'][0],
-                  'DATEQ': datetime.datetime.now().strftime('%d%m%Y%H%M%S'),
+        params = {
+            'VERSION': PAYBOX_DIRECT_VERSION_NUMBER,
+            'TYPE': operation,
+            'SITE': force_text(self.site),
+            'RANG': self.rang.strip(),
+            'CLE': force_text(self.cle),
+            'NUMQUESTION': bank_data['numero_transaction'][0].zfill(10),
+            'MONTANT': (amount * Decimal(100)).to_integral_value(ROUND_DOWN),
+            'DEVISE': force_text(self.devise),
+            'NUMTRANS': bank_data['numero_transaction'][0],  # paybox transaction number
+            'NUMAPPEL': bank_data['numero_appel'][0],
+            'REFERENCE': bank_data['reference'][0],
+            'DATEQ': datetime.datetime.now().strftime('%d%m%Y%H%M%S'),
         }
         response = requests.post(url, params)
         response.raise_for_status()
